#include "MF5p2.h"
#include "outpar.h"

//------------------------------------------------------------------------ 
CMF5p2::CMF5p2 ()
//------------------------------------------------------------------------ 
{
  int i; 
  ntyparr = 0;
  LINESIZE=1000;
  RR_RAD2DEG = 45.0  / atan( 1.0 );

// --- [DIMENSION]

   UNLOADED_RADIUS   = 0.0; //        =  0.364            $typarr(  7)        $Free tyre radius

// --- [VERTICAL]

   FNOMIN  = 0.0;//                   =  5002.8 ;        //  $typarr(  14)       $Nominal wheel load

// --- [LONG_SLIP_RANGE]

	KPUMIN  = 0.0;//                   =  -0.35946;       //  $typarr(  23)       $Minimum valid wheel slip
	KPUMAX  = 0.0;//                   =  0.60792  ;      //  $typarr(  24)       $Maximum valid wheel slip

// --- [SLIP_ANGLE_RANGE]

   ALPMIN  = 0.0;//                   =  -0.26            $typarr(  25)       $Minimum valid slip angle
   ALPMAX  = 0.0;//                   =  0.26             $typarr(  26)       $Maximum valid slip angle

// --- [SCALING_COEFFICIENTS]

	LFZO    = 0.0;//                   =  1 ;             //  $typarr(  31)       $Scale factor of nominal (rated) load
	LCX     = 0.0;//                   =  1 ;             //  $typarr(  32)       $Scale factor of Fx shape factor
	LMUX    = 0.0;//                   =  1 ;             //  $typarr(  33)       $Scale factor of Fx peak friction coefficient
	LEX     = 0.0;//                   =  1 ;             //  $typarr(  34)       $Scale factor of Fx curvature factor
	LKX     = 0.0;//                   =  1 ;             //  $typarr(  35)       $Scale factor of Fx slip stiffness
	LHX     = 0.0;//                   =  1 ;             //  $typarr(  36)       $Scale factor of Fx horizontal shift
	LVX     = 0.0;//                   =  1 ;             //  $typarr(  37)       $Scale factor of Fx vertical shift
	LGAX    = 0.0;//                   =  1 ;             //  $typarr(  58)       $Scale factor of camber for Fx
	LCY     = 0.0;//                   =  1 ;             //  $typarr(  38)       $Scale factor of Fy shape factor
	LMUY    = 0.0;//                   =  1 ;             //  $typarr(  39)       $Scale factor of Fy peak friction coefficient
	LEY     = 0.0;//                   =  1 ;             //  $typarr(  40)       $Scale factor of Fy curvature factor
	LKY     = 0.0;//                   =  1 ;             //  $typarr(  41)       $Scale factor of Fy cornering stiffness
	LHY     = 0.0;//                   =  1 ;             //  $typarr(  42)       $Scale factor of Fy horizontal shift
	LVY     = 0.0;//                   =  1 ;             //  $typarr(  43)       $Scale factor of Fy vertical shift
	LGAY    = 0.0;//                   =  1 ;             //  $typarr(  44)       $Scale factor of camber for Fy
	LTR     = 0.0;//                   =  1 ;             //  $typarr(  45)       $Scale factor of Peak of pneumatic trail
	LRES    = 0.0;//                   =  1 ;             //  $typarr(  46)       $Scale factor for offset of residual torque
	LGAZ    = 0.0;//                   =  1 ;             //  $typarr(  47)       $Scale factor of camber for Mz
	LXAL    = 0.0;//                   =  1 ;             //  $typarr(  48)       $Scale factor of alpha influence on Fx
	LYKA    = 0.0;//                   =  1 ;             //  $typarr(  49)       $Scale factor of alpha influence on Fx
	LVYKA   = 0.0;//                   =  1 ;             //  $typarr(  50)       $Scale factor of kappa induced Fy
	LS      = 0.0;//                   =  1 ;             //  $typarr(  51)       $Scale factor of Moment arm of Fx
	LSGKP   = 0.0;//                   =  1 ;             //  $typarr(  52)       $Scale factor of Relaxation length of Fx
	LSGAL   = 0.0;//                   =  1 ;             //  $typarr(  53)       $Scale factor of Relaxation length of Fy
	LGYR    = 0.0;//                   =  1 ;             //  $typarr(  54)       $Scale factor of gyroscopic torque
	LMX     = 0.0;//                   =  1 ;             //  $typarr(  55)       $Scale factor of overturning couple
	LVMX    = 0.0;//                   =  1 ;             //  $typarr(  57)       $Scale factor of Mx vertical shift
   LMY     = 0.0;//                   =  1 ;             //  $typarr(  56)       $Scale factor of rolling resistance torque

// --- [LONGITUDINAL_COEFFICIENTS]

	PCX1    = 0.0;//                   =  1.6442        ; // $typarr(  61)       $Shape factor Cfx for longitudinal force
	PDX1    = 0.0;//                   =  1.345878327   ; // $typarr(  62)       $Longitudinal friction Mux at Fznom
	PDX2    = 0.0;//                   =  -0.090562299  ; // $typarr(  63)       $Variation of friction Mux with load
	PDX3    = 0.0;//                   =  0.1           ; // $typarr(  60)       $Variation of friction Mux with camber
	PEX1    = 0.0;//                   =  0.652385145   ; // $typarr(  64)       $Longitudinal curvature Efx at Fznom
	PEX2    = 0.0;//                   =  -0.199921951  ; // $typarr(  65)       $Variation of curvature Efx with load
	PEX3    = 0.0;//                   =  0.036490905   ; // $typarr(  66)       $Variation of curvature Efx with load squared
	PEX4    = 0.0;//                   =  -0.14313      ; // $typarr(  67)       $Factor in curvature Efx while driving
	PKX1    = 0.0;//                   =  31.88104816   ; // $typarr(  68)       $Longitudinal slip stiffness Kfx/Fz at Fznom
	PKX2    = 0.0;//                   =  0.356212892   ; // $typarr(  69)       $Variation of slip stiffness Kfx/Fz with load
	PKX3    = 0.0;//                   =  0.162127778   ; // $typarr(  70)       $Exponent in slip stiffness Kfx/Fz with load
	PHX1    = 0.0;//                   =  0             ; // $typarr(  71)       $Horizontal shift Shx at Fznom
	PHX2    = 0.0;//                   =  0             ; // $typarr(  72)       $Variation of shift Shx with load
	PVX1    = 0.0;//                   =  0             ; // $typarr(  73)       $Vertical shift Svx/Fz at Fznom
	PVX2    = 0.0;//                   =  0             ; // $typarr(  74)       $Variation of shift Svx/Fz with load
	RBX1    = 0.0;//                   =  17.409        ; // $typarr(  75)       $Slope factor for combined slip Fx reduction
	RBX2    = 0.0;//                   =  12.892        ; // $typarr(  76)       $Variation of slope Fx reduction with kappa
	RCX1    = 0.0;//                   =  1.0968        ; // $typarr(  77)       $Shape factor for combined slip Fx reduction
	REX1    = 0.0;//                   =  0             ; // $typarr(  82)       $Curvature factor of combined Fx
	REX2    = 0.0;//                   =  0             ; // $typarr(  83)       $Curvature factor of combined Fx with load
	RHX1    = 0.0;//                   =  0.0012757     ; // $typarr(  78)       $Shift factor for combined slip Fx reduction
	PTX1    = 0.0;//                   =  1.040379682   ; // $typarr(  79)       $Relaxation length SigKap0/Fz at Fznom
	PTX2    = 0.0;//                   =  0.011624356   ; // $typarr(  80)       $Variation of SigKap0/Fz with load
	PTX3    = 0.0;//                   =  -0.162127778  ; // $typarr(  81)       $Variation of SigKap0/Fz with exponent of load

// --- [OVERTURNING_COEFFICIENTS]
	
   QSX1     = 0.0;//                 =  0.0065623        $typarr(  86)
	QSX2     = 0.0;//                 =  1.2408           $typarr(  87)
	QSX3     = 0.0;//                 =  0.0050152        $typarr(  88)

// --- [LATERAL_COEFFICIENTS]

	PCY1     = 0.0;//                 =  1.593            $typarr(  91)       $Shape factor Cfy for lateral forces
	PDY1     = 0.0;//                 =  1.1509           $typarr(  92)       $Lateral friction Muy
	PDY2     = 0.0;//                 =  -0.19648         $typarr(  93)       $Variation of friction Muy with load
	PDY3     = 0.0;//                 =  -3.7778          $typarr(  94)       $Variation of friction Muy with squared camber
	PEY1     = 0.0;//                =  0.18261          $typarr(  95)       $Lateral curvature Efy at Fznom
	PEY2     = 0.0;//                 =  -0.47338         $typarr(  96)       $Variation of curvature Efy with load
	PEY3     = 0.0;//                 =  -0.50782         $typarr(  97)       $Zero order camber dependency of curvature Efy
	PEY4     = 0.0;//                 =  -3.1777          $typarr(  98)       $Variation of curvature Efy with camber
	PKY1     = 0.0;//                 =  -34.7094         $typarr(  99)       $Maximum value of stiffness Kfy/Fznom
	PKY2     = 0.0;//                 =  2.2894           $typarr(  100)      $Load at which Kfy reaches maximum value
	PKY3     = 0.0;//                 =  1.61             $typarr(  101)      $Variation of Kfy/Fznom with camber
	PHY1     = 0.0;//                 =  0.0029549        $typarr(  102)      $Horizontal shift Shy at Fznom
	PHY2     = 0.0;//                 =  0.0014017        $typarr(  103)      $Variation of shift Shy with load
	PHY3     = 0.0;//                 =  0.057788         $typarr(  104)      $Variation of shift Shy with camber
	PVY1     = 0.0;//                 =  0.015785         $typarr(  105)      $Vertical shift in Svy/Fz at Fznom
	PVY2     = 0.0;//                 =  0.019288         $typarr(  106)      $Variation of shift Svy/Fz with load
	PVY3     = 0.0;//                 =  -0.31655         $typarr(  107)      $Variation of shift Svy/Fz with camber
	PVY4     = 0.0;//                 =  -0.27324         $typarr(  108)      $Variation of shift Svy/Fz with camber and load
	RBY1     = 0.0;//                 =  20.57            $typarr(  109)      $Slope factor for combined Fy reduction
	RBY2     = 0.0;//                 =  -23.533          $typarr(  110)      $Variation of slope Fy reduction with alpha
	RBY3     = 0.0;//                 =  0.0010504        $typarr(  111)      $Shift term for alpha in slope Fy reduction
	RCY1     = 0.0;//                 =  1.0094           $typarr(  112)      $Shape factor for combined Fy reduction
	REY1     = 0.0;//                 =  0                $typarr(  122)      $Curvature factor of combined Fy
	REY2     = 0.0;//                 =  0                $typarr(  123)      $Curvature factor of combined Fy with load
	RHY1     = 0.0;//                 =  -0.019642        $typarr(  113)      $Shift factor for combined Fy reduction
	RHY2     = 0.0;//                 =  0                $typarr(  124)      $Shift factor for combined Fy reduction with load
	RVY1     = 0.0;//                 =  0.15504          $typarr(  114)      $Kappa induced side force Svyk/Muy*Fz at Fznom
	RVY2     = 0.0;//                 =  0.035244199      $typarr(  115)      $Variation of Svyk/Muy*Fz with load
	RVY3     = 0.0;//                 =  27.548           $typarr(  116)      $Variation of Svyk/Muy*Fz with camber
	RVY4     = 0.0;//                =  -29.67           $typarr(  117)       $Variation of Svyk/Muy*Fz with alpha
	RVY5     = 0.0;//                 =  0.033819         $typarr(  118)      $Variation of Svyk/Muy*Fz with kappa
	RVY6     = 0.0;//                 =  0                $typarr(  119)      $Variation of Svyk/Muy*Fz with atan(kappa)
	PTY1     = 0.0;//                 =  1.8672           $typarr(  120)      $Peak value of relaxation length SigAlp0/R0
	PTY2     = 0.0;//                 =  2.2894           $typarr(  121)      $Value of Fz/Fznom where SigAlp0 is extreme

// --- [ROLLING_COEFFICIENTS]

	QSY1     = 0.0;//                 =  0.01             $typarr(  126)      $Rolling resistance torque coefficient
	QSY2     = 0.0;//                 =  0                $typarr(  127)      $Rolling resistance torque depending on Fx
	QSY3     = 0.0;//                 =  0                $typarr(  128)      $Rolling resistance torque depending on speed
	QSY4     = 0.0;//                 =  0                $typarr(  129)      $Rolling resistance torque depending on speed ^4

// --- [ALIGNING_COEFFICIENTS]

	QBZ1     = 0.0;//                 =  23.3412          $typarr(  131)      $Trail slope factor for trail Bpt at Fznom
	QBZ2     = 0.0;//                 =  -16.4228         $typarr(  132)      $Variation of slope Bpt with load
	QBZ3     = 0.0;//                 =  4.2624           $typarr(  133)      $Variation of slope Bpt with load squared
	QBZ4     = 0.0;//                 =  -1.4478          $typarr(  134)      $Variation of slope Bpt with camber
	QBZ5     = 0.0;//                 =  -6.1694          $typarr(  135)      $Variation of slope Bpt with absolute camber
	QBZ9     = 0.0;//                 =  34.7863          $typarr(  136)      $Slope factor Br of residual torque Mzr
	QBZ10    = 0.0;//                 =  0                $typarr(  130)      $Slope factor Br of residual torque Mzr
	QCZ1     = 0.0;//                =  1.0797            $typarr(  137)      $Shape factor Cpt for pneumatic trail
	QDZ1     = 0.0;//                 =  0.11531          $typarr(  138)      $Peak trail Dpt"
	QDZ2     = 0.0;//                 =  0.00095038       $typarr(  139)      $Variation of peak Dpt" with load
	QDZ3     = 0.0;//                 =  1.4096           $typarr(  140)      $Variation of peak Dpt" with camber
	QDZ4     = 0.0;//                 =  -217.4156        $typarr(  141)      $Variation of peak Dpt" with camber squared
	QDZ6     = 0.0;//                 =  -0.000038527     $typarr(  142)      $Peak residual torque Dmr"
	QDZ7     = 0.0;//                 =  0.0017346        $typarr(  143)      $Variation of peak factor Dmr" with load
	QDZ8     = 0.0;//                 =  -0.47743         $typarr(  144)      $Variation of peak factor Dmr" with camber
	QDZ9     = 0.0;//                 =  -0.11866         $typarr(  145)      $Variation of peak factor Dmr" with camber and load
	QEZ1     = 0.0;//                 =  -0.63536         $typarr(  146)      $Trail curvature Ept at Fznom
	QEZ2     = 0.0;//                 =  -2.7628          $typarr(  147)      $Variation of curvature Ept with load
	QEZ3     = 0.0;//                =  -1.9926           $typarr(  148)      $Variation of curvature Ept with load squared
	QEZ4     = 0.0;//                 =  0.39793          $typarr(  149)      $Variation of curvature Ept with sign of Alpha-t
	QEZ5     = 0.0;//                 =  -0.89939         $typarr(  150)      $Variation of Ept with camber and sign Alpha-t
	QHZ1     = 0.0;//                 =  0.0056093        $typarr(  151)      $Trail horizontal shift Sht at Fznom
	QHZ2     = 0.0;//                 =  -0.015899        $typarr(  152)      $Variation of shift Sht with load
	QHZ3     = 0.0;//                 =  0.36592          $typarr(  153)      $Variation of shift Sht with camber
	QHZ4     = 0.0;//                =  -0.83995          $typarr(  154)      $Variation of shift Sht with camber and load
	SSZ1     = 0.0;//                =  -0.011020922      $typarr(  155)      $Nominal value of s/R0: effect of Fx on Mz
	SSZ2     = 0.0;//                 =  0.053591208      $typarr(  156)      $Variation of distance s/R0 with Fy/Fznom
	SSZ3     = 0.0;//                 =  2.122838489      $typarr(  157)      $Variation of distance s/R0 with camber
	SSZ4     = 0.0;//                 =  -0.880324104     $typarr(  158)      $Variation of distance s/R0 with load and camber
	QTZ1     = 0.0;//                 =  0                $typarr(  159)      $Gyration torque constant
	MBELT    = 0.0;//                 =  0                $typarr(  160)      $Belt mass of the wheel

	SLMAX    = 0.0;
	SAMAX    = 0.0;
	SLINC    = 1.0;
	SAINC    = 1.0;

	for (i=0; i<NWheelLoads; i++)
	{
		Array_AC[i]    =    0;
		Array_NU[i]    =   50;
		Array_FZ[i]    = 1000.0*(i+1);
		Array_SA[i][0] =   20.0;
		Array_SL[i][0] =   20.0;
		Array_CA[i]    =    0.0;
	}
	Array_AC[1] = 1;
	Array_AC[3] = 1;
	Array_AC[5] = 1;
}
//--------------------------------------------------------------------------------------------------
CMF5p2::~CMF5p2()  
//--------------------------------------------------------------------------------------------------
{
}
//--------------------------------------------------------------------------------------------------
 void CMF5p2::get_typarr( char *lfn )
//--------------------------------------------------------------------------------------------------
{
   FILE *fp;
   char line[1000];
   char *sep;
   char my_varname[1000];
   char cvalue[1000];
   int  iarray=0;
   if ( ( fp = fopen( lfn,"r+" ) ) != NULL )
   {
      while ( fgets ( line,LINESIZE,fp ) != ( char * )0 )
      {
         if ( strstr( line, "$typarr(" ) )
         {
            iarray++;
            int i = 0;
            sep = line;
            sep=strtok( sep, "=$() " );
            sscanf( sep,"%s" , &my_varname );
            while ( ( sep=strtok( NULL, "=$() " ) ) != NULL )
            {
               i++;
               if ( i == 1 ) sscanf( sep,"%s" , &cvalue );
               if ( i == 3 ) sscanf( sep,"%d" , &iarray );
            }
            sscanf( my_varname,"%s"  , varname[iarray] );
            sscanf( cvalue,    "%lf" , &typarr[iarray] );
         }

      }
      ntyparr = iarray;
      fclose ( fp );
   }
}
//--------------------------------------------------------------------------------------------------
void CMF5p2::read_values( void )
//--------------------------------------------------------------------------------------------------
{
   int i = 0;
   while ( i <= ntyparr  )
   {
      i++;
      if ( strstr ( varname[i], "UNLOADED_RADIUS" ) ) UNLOADED_RADIUS = typarr[i];
      if ( strstr ( varname[i], "FNOMIN" ) ) FNOMIN = typarr[i];

      if ( strstr ( varname[i], "KPUMIN" ) ) KPUMIN = typarr[i];
      if ( strstr ( varname[i], "KPUMAX" ) ) KPUMAX = typarr[i];

      if ( strstr ( varname[i], "ALPMIN"    ) ) ALPMIN = typarr[i];
      if ( strstr ( varname[i], "ALPMAX"    ) ) ALPMAX = typarr[i];

      if ( strstr ( varname[i], "LFZO"   ) ) LFZO = typarr[i];
      if ( strstr ( varname[i], "LCX"    ) ) LCX = typarr[i];
      if ( strstr ( varname[i], "LMUX"   ) ) LMUX = typarr[i];
      if ( strstr ( varname[i], "LEX"    ) ) LEX = typarr[i];
      if ( strstr ( varname[i], "LKX"    ) ) LKX = typarr[i];
      if ( strstr ( varname[i], "LHX"    ) ) LHX = typarr[i];
      if ( strstr ( varname[i], "LVX"    ) ) LVX = typarr[i];
      if ( strstr ( varname[i], "LGAX"   ) ) LGAX = typarr[i];
      if ( strstr ( varname[i], "LCY"    ) ) LCY = typarr[i];
      if ( strstr ( varname[i], "LMUY"   ) ) LMUY = typarr[i];
      if ( strstr ( varname[i], "LEY"    ) ) LEY = typarr[i];
      if ( strstr ( varname[i], "LKY"    ) ) LKY = typarr[i];
      if ( strstr ( varname[i], "LHY"    ) ) LHY = typarr[i];
      if ( strstr ( varname[i], "LVY"    ) ) LVY = typarr[i];
      if ( strstr ( varname[i], "LGAY"   ) ) LGAY = typarr[i];
      if ( strstr ( varname[i], "LTR"    ) ) LTR = typarr[i];
      if ( strstr ( varname[i], "LRES"   ) ) LRES = typarr[i];
      if ( strstr ( varname[i], "LGAZ"   ) ) LGAZ = typarr[i];
      if ( strstr ( varname[i], "LXAL"   ) ) LXAL = typarr[i];
      if ( strstr ( varname[i], "LYKA"   ) ) LYKA = typarr[i];
      if ( strstr ( varname[i], "LVYKA"  ) ) LVYKA = typarr[i];
      if ( strstr ( varname[i], "LS"     ) ) LS = typarr[i];
      if ( strstr ( varname[i], "LSGKP"  ) ) LSGKP = typarr[i];
      if ( strstr ( varname[i], "LSGAL"  ) ) LSGAL = typarr[i];
      if ( strstr ( varname[i], "LGYR"   ) ) LGYR = typarr[i];
      if ( strstr ( varname[i], "LMX"    ) ) LMX = typarr[i];
      if ( strstr ( varname[i], "LVMX"   ) ) LVMX = typarr[i];
      if ( strstr ( varname[i], "LMY"    ) ) LMY = typarr[i];

      if ( strstr ( varname[i], "PCX1"    ) ) PCX1 = typarr[i];
      if ( strstr ( varname[i], "PDX1"    ) ) PDX1 = typarr[i];
      if ( strstr ( varname[i], "PDX2"    ) ) PDX2 = typarr[i];
      if ( strstr ( varname[i], "PDX3"    ) ) PDX3 = typarr[i];
      if ( strstr ( varname[i], "PEX1"    ) ) PEX1 = typarr[i];
      if ( strstr ( varname[i], "PEX2"    ) ) PEX2 = typarr[i];
      if ( strstr ( varname[i], "PEX3"    ) ) PEX3 = typarr[i];
      if ( strstr ( varname[i], "PEX4"    ) ) PEX4 = typarr[i];
      if ( strstr ( varname[i], "PKX1"    ) ) PKX1 = typarr[i];
      if ( strstr ( varname[i], "PKX2"    ) ) PKX2 = typarr[i];
      if ( strstr ( varname[i], "PKX3"    ) ) PKX3 = typarr[i];
      if ( strstr ( varname[i], "PHX1"    ) ) PHX1 = typarr[i];
      if ( strstr ( varname[i], "PHX2"    ) ) PHX2 = typarr[i];
      if ( strstr ( varname[i], "PVX1"    ) ) PVX1 = typarr[i];
      if ( strstr ( varname[i], "PVX2"    ) ) PVX2 = typarr[i];
      if ( strstr ( varname[i], "RBX1"    ) ) RBX1 = typarr[i];
      if ( strstr ( varname[i], "RBX2"    ) ) RBX2 = typarr[i];
      if ( strstr ( varname[i], "RCX1"    ) ) RCX1 = typarr[i];
      if ( strstr ( varname[i], "REX1"    ) ) REX1 = typarr[i];
      if ( strstr ( varname[i], "REX2"    ) ) REX2 = typarr[i];
      if ( strstr ( varname[i], "RHX1"    ) ) RHX1 = typarr[i];
      if ( strstr ( varname[i], "PTX1"    ) ) PTX1 = typarr[i];
      if ( strstr ( varname[i], "PTX2"    ) ) PTX2 = typarr[i];
      if ( strstr ( varname[i], "PTX3"    ) ) PTX3 = typarr[i];

// --- [OVERTURNING_COEFFICIENTS]

      if ( strstr ( varname[i], "QSX1"    ) ) QSX1 = typarr[i];
      if ( strstr ( varname[i], "QSX2"    ) ) QSX2 = typarr[i];
      if ( strstr ( varname[i], "QSX3"    ) ) QSX3 = typarr[i];
 
// --- [LATERAL_COEFFICIENTS]

      if ( strstr ( varname[i], "PCY1"    ) ) PCY1 = typarr[i];
      if ( strstr ( varname[i], "PDY1"    ) ) PDY1 = typarr[i];
      if ( strstr ( varname[i], "PDY2"    ) ) PDY2 = typarr[i];
      if ( strstr ( varname[i], "PDY3"    ) ) PDY3 = typarr[i];
      if ( strstr ( varname[i], "PEY1"    ) ) PEY1 = typarr[i];
      if ( strstr ( varname[i], "PEY2"    ) ) PEY2 = typarr[i];
      if ( strstr ( varname[i], "PEY3"    ) ) PEY3 = typarr[i];
      if ( strstr ( varname[i], "PEY4"    ) ) PEY4 = typarr[i];
      if ( strstr ( varname[i], "PKY1"    ) ) PKY1 = typarr[i];
      if ( strstr ( varname[i], "PKY2"    ) ) PKY2 = typarr[i];
      if ( strstr ( varname[i], "PKY3"    ) ) PKY3 = typarr[i];
      if ( strstr ( varname[i], "PHY1"    ) ) PHY1 = typarr[i];
      if ( strstr ( varname[i], "PHY2"    ) ) PHY2 = typarr[i];
      if ( strstr ( varname[i], "PHY3"    ) ) PHY3 = typarr[i];
      if ( strstr ( varname[i], "PVY1"    ) ) PVY1 = typarr[i];
      if ( strstr ( varname[i], "PVY2"    ) ) PVY2 = typarr[i];
      if ( strstr ( varname[i], "PVY3"    ) ) PVY3 = typarr[i];
      if ( strstr ( varname[i], "PVY4"    ) ) PVY4 = typarr[i];
      if ( strstr ( varname[i], "RBY1"    ) ) RBY1 = typarr[i];
      if ( strstr ( varname[i], "RBY2"    ) ) RBY2 = typarr[i];
      if ( strstr ( varname[i], "RBY3"    ) ) RBY3 = typarr[i];
      if ( strstr ( varname[i], "RCY1"    ) ) RCY1 = typarr[i];
      if ( strstr ( varname[i], "REY1"    ) ) REY1 = typarr[i];
      if ( strstr ( varname[i], "REY2"    ) ) REY2 = typarr[i];
      if ( strstr ( varname[i], "RHY1"    ) ) RHY1 = typarr[i];
      if ( strstr ( varname[i], "RHY2"    ) ) RHY2 = typarr[i];
      if ( strstr ( varname[i], "RVY1"    ) ) RVY1 = typarr[i];
      if ( strstr ( varname[i], "RVY2"    ) ) RVY2 = typarr[i];
      if ( strstr ( varname[i], "RVY3"    ) ) RVY3 = typarr[i];
      if ( strstr ( varname[i], "RVY4"    ) ) RVY4 = typarr[i];
      if ( strstr ( varname[i], "RVY5"    ) ) RVY5 = typarr[i];
      if ( strstr ( varname[i], "RVY6"    ) ) RVY6 = typarr[i];
      if ( strstr ( varname[i], "PTY1"    ) ) PTY1 = typarr[i];
      if ( strstr ( varname[i], "PTY2"    ) ) PTY2 = typarr[i];
    
// --- [ROLLING_COEFFICIENTS]

      if ( strstr ( varname[i], "QSY1"    ) ) QSY1 = typarr[i];
      if ( strstr ( varname[i], "QSY2"    ) ) QSY2 = typarr[i];
      if ( strstr ( varname[i], "QSY3"    ) ) QSY3 = typarr[i];
      if ( strstr ( varname[i], "QSY4"    ) ) QSY4 = typarr[i];
    
// --- [ALIGNING_COEFFICIENTS]

      if ( strstr ( varname[i], "QBZ1"    ) ) QBZ1 = typarr[i];
      if ( strstr ( varname[i], "QBZ2"    ) ) QBZ2 = typarr[i];
      if ( strstr ( varname[i], "QBZ3"    ) ) QBZ3 = typarr[i];
      if ( strstr ( varname[i], "QBZ4"    ) ) QBZ4 = typarr[i];
      if ( strstr ( varname[i], "QBZ5"    ) ) QBZ5 = typarr[i];
      if ( strstr ( varname[i], "QBZ9"    ) ) QBZ9 = typarr[i];
      if ( strstr ( varname[i], "QBZ10"   ) ) QBZ10 = typarr[i];
      if ( strstr ( varname[i], "QCZ1"    ) ) QCZ1 = typarr[i];
      if ( strstr ( varname[i], "QDZ1"    ) ) QDZ1 = typarr[i];
      if ( strstr ( varname[i], "QDZ2"    ) ) QDZ2 = typarr[i];
      if ( strstr ( varname[i], "QDZ3"    ) ) QDZ3 = typarr[i];
      if ( strstr ( varname[i], "QDZ4"    ) ) QDZ4 = typarr[i];
      if ( strstr ( varname[i], "QDZ6"    ) ) QDZ6 = typarr[i];
      if ( strstr ( varname[i], "QDZ7"    ) ) QDZ7 = typarr[i];
      if ( strstr ( varname[i], "QDZ8"    ) ) QDZ8 = typarr[i];
      if ( strstr ( varname[i], "QDZ9"    ) ) QDZ9 = typarr[i];
      if ( strstr ( varname[i], "QEZ1"    ) ) QEZ1 = typarr[i];
      if ( strstr ( varname[i], "QEZ2"    ) ) QEZ2 = typarr[i];
      if ( strstr ( varname[i], "QEZ3"    ) ) QEZ3 = typarr[i];
      if ( strstr ( varname[i], "QEZ4"    ) ) QEZ4 = typarr[i];
      if ( strstr ( varname[i], "QEZ5"    ) ) QEZ5 = typarr[i];
      if ( strstr ( varname[i], "QHZ1"    ) ) QHZ1 = typarr[i];
      if ( strstr ( varname[i], "QHZ2"    ) ) QHZ2 = typarr[i];
      if ( strstr ( varname[i], "QHZ3"    ) ) QHZ3 = typarr[i];
      if ( strstr ( varname[i], "QHZ4"    ) ) QHZ4 = typarr[i];
      if ( strstr ( varname[i], "SSZ1"    ) ) SSZ1 = typarr[i];
      if ( strstr ( varname[i], "SSZ2"    ) ) SSZ2 = typarr[i];
      if ( strstr ( varname[i], "SSZ3"    ) ) SSZ3 = typarr[i];
      if ( strstr ( varname[i], "SSZ4"    ) ) SSZ4 = typarr[i];
      if ( strstr ( varname[i], "QTZ1"    ) ) QTZ1 = typarr[i];
      if ( strstr ( varname[i], "MBELT"   ) ) MBELT = typarr[i];
   }
}
//-------------------------------------------------------------------------------------------------
void CMF5p2::calc_Fx( int inumberx, int CurveNumber )
//-------------------------------------------------------------------------------------------------
{
	int i;

   double fz0 = FNOMIN;

   double Fx0,Dx,Cx,Ex,Bx,dfz,Fz0,Fz0_der;
   double kx,k,mux,sign_kx;
   double Shx,Svx,Kx,gamma,gamma_x,sl;
   double Fzn;
   double tmp;
//
// --- Global
//
   for ( i=0; i<inumberx; i++ )
   {			
		sl = Array_SL[CurveNumber][0] - 2.0*Array_SL[CurveNumber][0]*((double) i)/((double) inumberx-1);

      k=sl*0.01;    
      if ( k < KPUMIN ) k = KPUMIN;
      if ( k > KPUMAX ) k = KPUMAX;
      Fzn=Array_FZ[CurveNumber];
      Fz0=fz0;  
      Fz0_der=LFZO*Fz0;
      dfz=( Fzn-Fz0_der )/Fz0_der;
      gamma=Array_CA[CurveNumber]/RR_RAD2DEG;
//
// --- Fx (pure slip)
//
      Shx=( PHX1+PHX2*dfz )*LHX;
      Svx=Fzn*( PVX1+PVX2*dfz )*LVX*LMUX;
      kx=k+Shx;
      if ( kx<0 )sign_kx=-1.0;
      else    sign_kx=1.0;

      gamma_x=gamma*LGAX;

      Cx=PCX1*LCX;
      mux=( PDX1+PDX2*dfz )*( 1.0-( PDX3*gamma_x*gamma_x ) )*LMUX;  
      Ex=( PEX1+PEX2*dfz+PEX3*dfz*dfz )*( 1.0-PEX4*sign_kx )*LEX;
      if ( Ex>1.0 )Ex=1.0;
      Dx=mux*Fzn;
      Kx=Fzn*( PKX1+PKX2*dfz )*exp( PKX3*dfz )*LKX;            
      tmp=Cx*Dx;
      if ( fabs( tmp )<0.001 )
      {
         Bx=Kx/( tmp+0.001 );
      }
      else
      {
         Bx=Kx/tmp;
      }

      tmp=Bx*kx;
      Fx0=Dx*sin( Cx*atan( tmp-Ex*( tmp-atan( tmp ) ) ) )+Svx;
      if ( i > 0 ) Array_SL [CurveNumber][i] = sl;
      Array_FX [CurveNumber][i] = Fx0;
   }
}
//-------------------------------------------------------------------------------------------------
void CMF5p2::calc_FyMz( int inumbery, int CurveNumber )
//-------------------------------------------------------------------------------------------------
{
//
// --- Fy
//
	int    i;

   double fz0 = FNOMIN;
   double sideSlip;
   double Fzn, Fz0, Fz0_der, dfz;

   double Fy0,Dy,Cy,Ey,By;
   double localMuy;
   double Shy,Svy,Ky;
   double sa,say,sign_alpha_y;
   double gamma_y, gamma;
   double alpha;
   double tmp,Mz;

   double Mzr;
   double t0,Dt,Ct,Bt,alpha_t,Et,gamma_z;
   double Sht,Shf;
   double Br,R0;
   double alpha_r,Dr;
   const float lr=1.0;        

   for ( i=0; i<inumbery; i++ )
   {
//
// --- Fy
//
		sa = Array_SA[CurveNumber][0] - 2.0*Array_SA[CurveNumber][0]*((double) i)/((double) inumbery-1);
		sideSlip = sa;
      Fzn=Array_FZ[CurveNumber];
      Fz0=fz0; 
      Fz0_der=LFZO*Fz0;
      dfz=( Fzn-Fz0_der )/Fz0_der;
      gamma=Array_CA[CurveNumber]/RR_RAD2DEG;

      alpha=sideSlip/RR_RAD2DEG;
      if ( alpha < ALPMIN ) alpha = ALPMIN;
      if ( alpha > ALPMAX ) alpha = ALPMAX;
      gamma_y=gamma*LGAY;

      Cy=PCY1*LCY;
      localMuy=( PDY1+PDY2*dfz )*( 1.0-PDY3*gamma_y*gamma_y )*LMUY;
      Dy=localMuy*Fzn;
      Ky=PKY1*Fz0_der*sin( 2.0*atan( Fzn/( PKY2*Fz0_der ) ) )*( 1.0-PKY3*fabs( gamma_y ) )*LFZO*LKY;  

      tmp=Cy*Dy;
      if ( fabs( tmp )<0.001 )
         By=Ky/( tmp+0.001 );
      else
         By=Ky/tmp;

      Shy=( PHY1+PHY2*dfz )*LHY+PHY3*gamma_y;
      Svy=Fzn*( ( PVY1+PVY2*dfz )*LVY+( PVY3+PVY4*dfz )*gamma_y )*LMUY;
      say=alpha+Shy;

      if ( say<0 )sign_alpha_y=-1.0;
      else     sign_alpha_y=1.0;
      Ey=( PEY1+PEY2*dfz )*( 1.0-( PEY3+PEY4*gamma_y )*sign_alpha_y )*LEY;
      if ( Ey>1.0 )Ey=1.0;

      tmp=By*say;
      Fy0=Dy*sin( Cy*atan( tmp-Ey*( tmp-atan( tmp ) ) ) )+Svy;
//
// --- Mz
//
      R0=UNLOADED_RADIUS;  
      gamma_z=gamma*LGAZ;

      Sht=QHZ1+QHZ2*dfz+( QHZ3+QHZ4*dfz )*gamma_z;
      alpha_t=alpha*( 1.0 )+Sht;

      if ( fabs( Ky )<0.001f )
		{
         if ( Ky < 0 ) 
			{
				Ky=-0.001f;
			}
         else    
			{
				Ky=0.001f;
			}
		}

      Shf=Shy+Svy/Ky;
      alpha_r=alpha+Shf;
      Bt=( QBZ1+QBZ2*dfz+QBZ3*dfz*dfz )*( 1.0+QBZ4*gamma_z+QBZ5*fabs( gamma_y ) )*LKY/LMUY;  
      Ct=QCZ1;
      Dt=Fzn*( QDZ1+QDZ2*dfz )*( 1.0+QDZ3*gamma_z+QDZ4*gamma_z*gamma_z )*( R0/Fz0_der )*LTR;   

      Et=( QEZ1+QEZ2*dfz+QEZ3*dfz*dfz )*( 1.0+( QEZ4+QEZ5*gamma_z )*( 2.0/3.14159265f )*atan( Bt*Ct*alpha_t ) );  

      if ( Et > 1.0 ) Et=1.0;

      Br=QBZ9*LKY/LMUY+QBZ10*By*Cy;      
      tmp=Bt*alpha_t;
      t0=Dt*cos( Ct*atan( tmp-Et*( tmp-atan( tmp ) ) ) )*cos( alpha );  
      Dr=Fzn*( ( QDZ6+QDZ7*dfz )*lr+( QDZ8+QDZ9*dfz )*gamma_z )*R0*LMUY;   
      Mzr=Dr*cos( Ct*atan( Br*alpha_r ) )*cos( alpha );                 

// --- No combined aligning moment

      Mz=-t0*Fy0+Mzr;
      if ( i > 0 ) Array_SA [CurveNumber][i] = sa; 
      Array_FY [CurveNumber][i] = -Fy0;
      Array_MZ [CurveNumber][i] = Mz*1000.0;  
   }
}
//---------------------------------------------------------------------------------------------------------
void CMF5p2::TDX_HEADER ( int UNIT, int ICLOSE, FILE *myfile )
//---------------------------------------------------------------------------------------------------------
{
//----------------------------------------------------------------------
//     WRITE TDX HEADER TO RESULT FILE
//     DATA ARE WRITTEN IN FBINFO
//----------------------------------------------------------------------

   int N_CHC = 42; 
   int N_CHA = 43;  
   int IVX,FZSOLL,ICTRL;
   int II;

   char CH_HEADER[2][73];
   char CH_PLOTTITLE[2][73];
   char CH_COM[43][73];
   char CH_MEA[44][73];
   char SIMTYPE[21];
   char CHTEXT[12];
   char CHTEXT62[51];
   char CH_UNIT[101],CH_UNITM[101],CH_UNITRTD[101];

   double RDUM, DUMMY_OFFSET, RDUM_H;
   strcpy ( CH_HEADER[ 0],"**PLOTHEADER                                                  " );
   strcpy ( CH_PLOTTITLE[ 0],"**PLOTTITLE                                                   " );

   strcpy ( CH_COM[ 1],"**CONSTANTS                                                   " );
   strcpy ( CH_COM[ 2],"PRO_NR    gedas_project_number          -         696         " );
   strcpy ( CH_COM[ 3],"NOMWIDTH  nominal_section_width_of_tire mm        225         " );
   strcpy ( CH_COM[ 4],"ASPRATIO  nominal_aspect_ratio          %         50          " );
   strcpy ( CH_COM[ 5],"TYSTRUCT  tyre_structure                -         R           " );
   strcpy ( CH_COM[ 6],"RIMDIAME  nominal_rim_diameter          inch      17          " );
   strcpy ( CH_COM[ 7],"RIMWIDTH  rim_width                     inch      7.5         " );
   strcpy ( CH_COM[ 8],"MANUFACT  brand_name                    -         Continstone " );
   strcpy ( CH_COM[ 9],"IDENTITY  commercial_name               -         SC2_98Y     " );
   strcpy ( CH_COM[10],"LOAD      vertical_load                 N         3201        " );
   strcpy ( CH_COM[11],"INFLPRES  inflation_pressure            bar       2.21        " );
   strcpy ( CH_COM[12],"TRAJVELC  velocity                      km/h      21          " );
   strcpy ( CH_COM[13],"CL_HEIGHT Cleat_height                  mm        6.1         " );
   strcpy ( CH_COM[14],"CL_LENGHT Cleat_length                  mm        25.1        " );
   strcpy ( CH_COM[15],"CHAMFORX  Chaphor_X_Length              mm        1.1         " );
   strcpy ( CH_COM[16],"CHAMFORY  Chaphor_Y_Length              mm        1.1         " );
   strcpy ( CH_COM[17],"DRUMRAD   Drum_radius                   mm        1590.2      " );
   strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
   strcpy ( CH_COM[19],"RESYID1   Result_y_ID_1                 -         4           " );
   strcpy ( CH_COM[20],"RESYID2   Result_y_ID_2                 -         2           " );
   strcpy ( CH_COM[21],"RESYID3   Result_y_ID_3                 -         3           " );
   strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
   strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         none        " );
   strcpy ( CH_COM[24],"RESXTIME  Result_x_ID_Time              s         1           " );
   strcpy ( CH_COM[25],"RESYFX    Result_x_ID_FX                N         2           " );
   strcpy ( CH_COM[26],"RESYFY    Result_x_ID_FY                N         3           " );
   strcpy ( CH_COM[27],"RESYFZ    Result_x_ID_FZ                N         4           " );
   strcpy ( CH_COM[28],"RESMX     Result_x_ID_MX                Nmm       5           " );
   strcpy ( CH_COM[29],"RESMY     Result_x_ID_MY                Nmm       6           " );
   strcpy ( CH_COM[30],"RESMZ     Result_x_ID_MZ                Nmm       7           " );
   strcpy ( CH_COM[31],"RESSA     Result_x_ID_SideSlipAngle     deg       14          " );
   strcpy ( CH_COM[32],"RESLS     Result_x_ID_LongitudinalSlip  %         15          " );
   strcpy ( CH_COM[33],"RESXDEFL  Result_x_ID_VerticalDeflec    mm        16          " );
   strcpy ( CH_COM[34],"CLEATANG  Cleat_Angle                   deg       17          " );
   strcpy ( CH_COM[35],"VERT_STI  linear_vertical_stiffness     N/mm      18          " );
   strcpy ( CH_COM[36],"WL_MAG    WheelLoadMagnitude            N,mm      19          " );
   strcpy ( CH_COM[37],"WL_FRE    WheelLoadFrequency            Hz        20          " );
   strcpy ( CH_COM[38],"RIM_DISP  RimDisplacementZ              mm        21          " );
   strcpy ( CH_COM[39],"WL_STAT   WheelLoad                     N         22          " );
   strcpy ( CH_COM[40],"WL_CTR    WheelLoadControl              -         23          " );
   strcpy ( CH_COM[41],"WL_DYN    WheelLoadDynamic              -         24          " );
   strcpy ( CH_COM[42],"                                                              " );

   strcpy ( CH_MEA[ 1],"**MEASURCHANNELS                                 " );
   if ( OUTPAR [M_SIM] ==  3.0e0 )
   {
      strcpy ( CH_MEA[ 2],"SL        Longitudinal_slip             %        " );
      strcpy ( CH_MEA[ 3],"FX        Longitudinal_Force            N        " );
      strcpy ( CH_MEA[ 4],"**MEASURDATA                                     " );
      strcpy ( CH_MEA[ 5],"**END                                            " );
   }
   else if ( OUTPAR [M_SIM] ==  4.0e0 )
   {
      strcpy ( CH_MEA[ 2],"DELTA     Side_Slip_angle               deg      " );
      strcpy ( CH_MEA[ 3],"FY        Lateral_Force                 N        " );
      strcpy ( CH_MEA[ 4],"MZ        Z_Moment                      Nmm      " );
      strcpy ( CH_MEA[ 5],"**MEASURDATA                                     " );
      strcpy ( CH_MEA[ 6],"**END                                            " );
   }
  
   strcpy ( CH_MEA[41],"**MEASURDATA                                     " );
   strcpy ( CH_MEA[42],"**END                                            " );

   strcpy ( CH_UNIT     ,"1         0         0         " );
   strcpy ( CH_UNITM    ,"-1        0         0         " );
   strcpy ( CH_UNITRTD  ,"57.3      0         0         " );

   if ( OUTPAR[M_SIM] == 1.0e0 )
   {
      strcpy ( SIMTYPE,"cleat     " );
   }
   else if ( OUTPAR [M_SIM] ==  2.0e0 )
   {
      strcpy ( SIMTYPE,"verts      " );
   }
   else if ( OUTPAR [M_SIM] ==  3.0e0 )
   {
      strcpy ( SIMTYPE,"braking   " );
   }
   else if ( OUTPAR [M_SIM] ==  4.0e0 )
   {
      strcpy ( SIMTYPE,"steering  " );
   }
   else if ( OUTPAR [M_SIM] ==  5.0e0 )
   {
      strcpy ( SIMTYPE,"monof      " );
   }
   else if ( OUTPAR [M_SIM] ==  6.0e0 )
   {
      strcpy ( SIMTYPE,"monod      " );
   }
   else if ( OUTPAR [M_SIM] ==  7.0e0 )
   {
      strcpy ( SIMTYPE,"sweepf     " );
   }
   else if ( OUTPAR [M_SIM] ==  8.0e0 )
   {
      strcpy ( SIMTYPE,"sweepd     " );
   }
   else
   {
      strcpy ( SIMTYPE,"undefined  " );
      OUTPAR [M_SIM] = 0.0e0;
   }

   if ( strstr ( SIMTYPE,"cleat" ) )
   {
      ICTRL = 1;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
      strcpy ( CHTEXT," " );

// --- CleatHeight

      RDUM_H  = ( OUTPAR [M_CLH] - OUTPAR [M_CCH] ) * 2.0e0;
      RDUM_H  = RDUM_H / ( OUTPAR [M_VXKM] * 1000.0e0 / 3.6e0 );

      DUMMY_OFFSET = -( TDX_XOFFSET - RDUM_H );
      sprintf ( CHTEXT,"%10.4f\0\n",( float ) DUMMY_OFFSET );
      strncpy ( &CH_COM[22][50],CHTEXT,11 );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         cleat       " );

   }
   else if ( strstr ( SIMTYPE,"verts" ) )
   {
      ICTRL = 1;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         16          " );
      strcpy ( CH_COM[22],"XOFFSET   Deflection_offset             mm        0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         vertstiff   " );

   }
   else if ( strstr ( SIMTYPE,"braki" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         2           " );
      strcpy ( CH_COM[20],"dESYID2   Result_y_ID2                  -         3           " );
      strcpy ( CH_COM[21],"dESYID3   Result_y_ID3                  -         4           " );
      strcpy ( CH_COM[22],"XOFFSET   Longitudinal_slip_offset      %         0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         braking     " );
      strcpy ( CH_COM[24],"dESXTIME  Result_x_ID_Time              s         1           " );
      strcpy ( CH_COM[25],"RESYFX    Result_x_ID_FX                N         2           " );
      strcpy ( CH_COM[26],"dESYFY    Result_x_ID_FY                N         3           " );
      strcpy ( CH_COM[27],"dESYFZ    Result_x_ID_FZ                N         4           " );
      strcpy ( CH_COM[28],"dESMX     Result_x_ID_MX                Nmm       5           " );
      strcpy ( CH_COM[29],"dESMY     Result_x_ID_MY                Nmm       6           " );
      strcpy ( CH_COM[30],"dESMZ     Result_x_ID_MZ                Nmm       7           " );
      strcpy ( CH_COM[31],"dESSA     Result_x_ID_SideSlipAngle     deg       14          " );
      strcpy ( CH_COM[32],"RESLS     Result_x_ID_LongitudinalSlip  %         1           " );
      strcpy ( CH_COM[33],"dESXDEFL  Result_x_ID_VerticalDeflec    mm        16          " );

   }
   else if ( strstr ( SIMTYPE,"steer" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         2           " );
      strcpy ( CH_COM[20],"RESYID2   Result_y_ID2                  -         3           " );
      strcpy ( CH_COM[21],"dESYID3   Result_y_ID3                  -         4           " );
      strcpy ( CH_COM[22],"XOFFSET   Slip_angle_offset             deg       0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         steering    " );
      strcpy ( CH_COM[24],"dESXTIME  Result_x_ID_Time              s         1           " );
      strcpy ( CH_COM[25],"dESYFX    Result_x_ID_FX                N         2           " );
      strcpy ( CH_COM[26],"RESYFY    Result_x_ID_FY                N         2           " );
      strcpy ( CH_COM[27],"dESYFZ    Result_x_ID_FZ                N         4           " );
      strcpy ( CH_COM[28],"dESMX     Result_x_ID_MX                Nmm       5           " );
      strcpy ( CH_COM[29],"dESMY     Result_x_ID_MY                Nmm       6           " );
      strcpy ( CH_COM[30],"RESMZ     Result_x_ID_MZ                Nmm       3           " );
      strcpy ( CH_COM[31],"RESSA     Result_x_ID_SideSlipAngle     deg       1           " );
      strcpy ( CH_COM[32],"dESLS     Result_x_ID_LongitudinalSlip  %         15          " );
      strcpy ( CH_COM[33],"dESXDEFL  Result_x_ID_VerticalDeflec    mm        16          " );
   }
   else if ( strstr ( SIMTYPE,"monof" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         4           " );
      strcpy ( CH_COM[20],"RESYID2   Result_y_ID2                  -         29          " );
      strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         monoforce   " );
      strcpy ( CH_COM[33],"RESXDEFL  Result_x_ID_VerticalDeflec    mm        39          " );
   }
   else if ( strstr ( SIMTYPE,"monod" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         4           " );
      strcpy ( CH_COM[20],"RESYID2   Result_y_ID2                  -         29          " );
      strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         monodisplace" );
   }
   else if ( strstr ( SIMTYPE,"sweepf" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         4           " );
      strcpy ( CH_COM[20],"RESYID2   Result_y_ID2                  -         29          " );
      strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         sweepforce  " );
   }
   else if ( strstr ( SIMTYPE,"sweepd" ) )
   {
      ICTRL = 2;
      strcpy ( CH_COM[18],"RESXID    Result_x_ID                   -         1           " );
      strcpy ( CH_COM[19],"RESYID1   Result_y_ID1                  -         4           " );
      strcpy ( CH_COM[20],"RESYID2   Result_y_ID2                  -         29          " );
      strcpy ( CH_COM[22],"XOFFSET   Time_offset                   sec       0.0         " );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         sweepdisplac" );
   }
   else
   {
      if ( L_TDI ) printf ( " Simulationtype incorrect : %s\n",SIMTYPE );
      strcpy ( CH_COM[23],"SIMTYPE   Kind_of_simulation            -         unknown     " );
   }

// --- VX

   IVX    = ( int ) OUTPAR [M_VXKM];
   sprintf ( CHTEXT,"%10d",IVX );
   strcpy ( &CH_COM[12][50],CHTEXT );

// --- LOAD

   FZSOLL    = ( int ) OUTPAR [M_LOAD];
   sprintf ( CHTEXT,"%10d",FZSOLL );
   strcpy ( &CH_COM[10][50],CHTEXT );

// --- PIN

   RDUM    = OUTPAR [M_PIN];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[11][50],CHTEXT,11 );

// --- CleatHeight

   RDUM    = OUTPAR [M_CLH];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[13][50],CHTEXT,11 );

// --- CleatLength

   RDUM    = OUTPAR [M_CLL];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[14][50],CHTEXT,11 );

// --- CleatCamphorHeight

   RDUM    = OUTPAR [M_CCH];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[15][50],CHTEXT,11 );

// --- CleatCamphorLength

   RDUM    = OUTPAR [M_CCL];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[16][50],CHTEXT,11 );

// --- Drumradius

   RDUM    = OUTPAR [M_DRA];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[17][50],CHTEXT,11 );

// --- CleatAngle

   RDUM    = OUTPAR [M_CANG];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[34][50],CHTEXT,11 );

// --- Vertical Stiffness

   RDUM    = OUTPAR [M_VERT];
   sprintf ( CHTEXT,"%10.4f",( float ) RDUM );
   strncpy ( &CH_COM[35][50],CHTEXT,11 );

// --- WheelLoadMagnitude

   sprintf ( CHTEXT,"%10d",OUTPAR [M_WLDM] );
   strcpy ( &CH_COM[36][50],CHTEXT );

// --- WheelLoadFrequency

   sprintf ( CHTEXT,"%10d",OUTPAR [M_WLF] );
   strcpy ( &CH_COM[37][50],CHTEXT );

// --- RimDisplacementZ

   sprintf ( CHTEXT,"%10d",OUTPAR [M_RDZ] );
   strcpy ( &CH_COM[38][50],CHTEXT );

// --- WheelLoad stat

   sprintf ( CHTEXT,"%10d",OUTPAR [M_LOAD] );
   strcpy ( &CH_COM[39][50],CHTEXT );

// --- WheelLoadControl

   sprintf ( CHTEXT,"%10d",OUTPAR [M_ILOAD] );
   strcpy ( &CH_COM[40][50],CHTEXT );

// --- WheelLoadDynamic

   sprintf ( CHTEXT,"%10d",OUTPAR [M_WLD] );
   strcpy ( &CH_COM[41][50],CHTEXT );

   char CHDUMMY[256];
   sprintf ( CHDUMMY   ,"%s\0",SIMTYPE );
   int j = 0;
   for ( int i=0; i < int ( strlen ( CHDUMMY ) ); i++ )
   {
      if ( CHDUMMY[i]!= ( int )' ' )
      {
         CH_HEADER  [1][j]=CHDUMMY[i];
         j++;
      }
   }
   sprintf ( &CH_HEADER  [1][j]   ,"\0" );
   char CHDUMMY1[256];
   int idum = ( int ) OUTPAR [M_LOAD];
   sprintf ( CHDUMMY1,"%s_V%dkmh_%dN\0",SIMTYPE,( int ) OUTPAR [M_VXKM],( int ) OUTPAR [M_LOAD] );
   j = 0;
   for ( int i=0; i < (int) (strlen ( CHDUMMY1 )); i++ )
   {
      if ( CHDUMMY1[i]!= ( int )' ' )
      {
         CH_PLOTTITLE  [1][j]=CHDUMMY1[i];
         j++;
      }
   }
   sprintf ( &CH_PLOTTITLE  [1][j]   ,"\0" );

   if ( ICLOSE == 0 )
   {
      fprintf ( myfile,"%s\n",CH_HEADER[0] );
      fprintf ( myfile,"%s\n",CH_HEADER[1] );
      fprintf ( myfile,"%s\n",CH_PLOTTITLE[0] );
      fprintf ( myfile,"%s\n",CH_PLOTTITLE[1] );

      for ( II = 1; II <=N_CHC; II++ )
      {
         if ( II > 1 && II <=9 )
         {
            if ( II == 2 )
            {
               strcpy ( CHTEXT62,"FILE      Input_File                     -        " );
               strcpy ( INPRFNFILE , IFILE1 ) ;
            }
         }
         else
         {
            fprintf ( myfile,"%s\n",CH_COM[II] );
         }
      }

// --- VERTSTIFF CLEAT

      if ( OUTPAR [M_SIM] ==  3.0e0 )
      {
         for ( II = 1; II <=5; II ++ )
         {
            if ( II==1 || II >3 ) fprintf ( myfile,"%s \n",CH_MEA[II] );
            else                  fprintf ( myfile,"%s %s\n",CH_MEA[II],CH_UNIT );
         }
      }
      else if ( OUTPAR [M_SIM] ==  4.0e0 )
      {
         for ( II = 1; II <=6; II ++ )
         {
            if ( II==1 || II >4 ) fprintf ( myfile,"%s \n",CH_MEA[II] );
            else                  fprintf ( myfile,"%s %s\n",CH_MEA[II],CH_UNIT );
         }
      }

// --- BRAKING STEERING

   }
   else if ( ICLOSE == 1 )
   {
      fprintf ( myfile,"%s \n",CH_MEA[N_CHA] );
   }

   return;
}
void CMF5p2::write_values()
{
	int  j,number;

   FILE *TARGET;
   FILE *OUTPUT;
	char OutName [512];
	double null=0.0;

	number = 0;
	for (j=0; j<NWheelLoads; j++ )
	{
		if ( Array_AC[j] == 1 ) number++;
	}

	if ( ( TARGET = fopen ("TargetMF.dat","w+") ) == NULL ) return;
  
   fprintf ( TARGET,"[DataFileName and Size]\n" );
	fprintf ( TARGET,"Number of Files = %d\n",2*number);


	number = 0;
	for (j=0; j<NWheelLoads; j++ )
	{
		if ( Array_AC[j] == 1 ) 
		{
//
// --- write long. data
//
			sprintf ( OutName,"MF_Fx_%d.dat",number+1);
			if ( ( OUTPUT = fopen (OutName,"w+") ) == NULL ) return;
			for ( int i = 0; i< Array_NU[j]; i++ )
				fprintf ( OUTPUT,"%+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e\n",Array_SL[j][i]/100.0,null,Array_CA[j],Array_FX[j][i],null,Array_FZ[j], null);
			fclose ( OUTPUT );
//
// --- write target data
//
		   fprintf ( TARGET,"Datafile%d = %s Size = %d dir = 1 VT = %lf\n",2*number+1,OutName,Array_NU[j],8000.0);
//
// --- write lat. data
//
			sprintf ( OutName,"MF_Fy_%d.dat",number+1);
			if ( ( OUTPUT = fopen (OutName,"w+") ) == NULL ) return;
			for ( int i = 0; i< Array_NU[j]; i++ )
				fprintf ( OUTPUT,"%+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e\n",null,Array_SA[j][i],Array_CA[j],null,Array_FY[j][i],Array_FZ[j],Array_MZ[j][i] );
			fclose ( OUTPUT );
//
// --- write target data
//
		   fprintf ( TARGET,"Datafile%d = %s Size = %d dir = 2 VT = %lf\n",2*number+2,OutName,Array_NU[j],8000.0);
			number++;
		}
	}
	fclose ( TARGET );
   return;
}
